@@ -16,12 +16,11 @@ use IPC::Cmd ();		# Core since 5.9.5
 use Module::Load::Conditional ();	# Core since 5.9.5
 use Pod::Perldoc ();		# Core since 5.8.1
 use Pod::Simple::LinkSection;	# Core since 5.9.3 (part of Pod::Simple)
-use Pod::Simple::SimpleTree ();	# Core since 5.9.3 (part of Pod::Simple)
 use Scalar::Util ();		# Core since 5.7.3
 use Storable ();		# Core since 5.7.3
 use Test::Builder ();		# Core since 5.6.2
 
-our $VERSION = '0.000_901';
+our $VERSION = '0.000_902';
 
 use constant ON_DARWIN		=> 'darwin' eq $^O;
 use constant ON_VMS		=> 'VMS' eq $^O;
@@ -78,6 +77,14 @@ sub new {
     }
 }
 
+sub _default_agent {
+    return HTTP::Tiny->new()->agent();
+}
+
+sub _default_check_external_sections {
+    return 1;
+}
+
 sub _default_check_url {
     return 1;
 }
@@ -113,8 +120,20 @@ sub _default_module_index {
     return \@handlers;
 }
 
-sub _default_agent {
-    return HTTP::Tiny->new()->agent();
+sub _default_require_installed {
+    return 0;
+}
+
+sub _init_agent {
+    my ( $self, $name, $value ) = @_;
+    $self->{$name} = $value;
+    return;
+}
+
+sub _init_check_external_sections {
+    my ( $self, $name, $value ) = @_;
+    $self->{$name} = $value ? 1 : 0;
+    return;
 }
 
 sub _init_check_url {
@@ -196,9 +215,9 @@ sub _init_module_index {
     return;
 }
 
-sub _init_agent {
+sub _init_require_installed {
     my ( $self, $name, $value ) = @_;
-    $self->{$name} = $value;
+    $self->{$name} = $value ? 1 : 0;
     return;
 }
 
@@ -209,6 +228,11 @@ sub _process_array_argument {
     return [ map { split qr< \s*, \s* >smx } @{ $arg } ];
 }
 
+sub agent {
+    my ( $self ) = @_;
+    return $self->{agent};
+}
+
 sub all_pod_files_ok {
     my ( $self, @dir ) = @_;
 
@@ -235,6 +259,11 @@ sub all_pod_files_ok {
     return wantarray ? ( $fail, $pass, $skip ) : $fail;
 }
 
+sub check_external_sections {
+    my ( $self ) = @_;
+    return $self->{check_external_sections}
+}
+
 sub check_url {
     my ( $self ) = @_;
     return $self->{check_url}
@@ -286,16 +315,12 @@ sub pod_file_ok {
 	skip	=> 0,
     };
 
-    my $parser = Pod::Simple::SimpleTree->new();
-
     if ( SCALAR_REF eq ref $file ) {
 	$self->{_file_name} = ${ $file } =~ m/ \n /smx ?
 	    "String $file" :
 	    "String '${ $file }'";
-	$parser->parse_string_document( ${ $file } );
     } elsif ( -f $file ) {
 	$self->{_file_name} = "File $file";
-	$parser->parse_file( $file );
     } else {
 	$self->{_file_name} = "File $file";
 	$self->_fail(
@@ -303,22 +328,21 @@ sub pod_file_ok {
 	return wantarray ? ( 1, 0, 0 ) : 1;
     }
 
-    $parser->any_errata_seen()
-	and $TEST->diag( "$self->{_file_name} contains POD errors" );
+    ( $self->{_section}, $self->{_links} ) = My_Parser->new()->run(
+	$file, \&_any_errata_seen, $self );
 
-    $self->{_root} = $parser->root();
-    my @links = $self->_extract_nodes( \&_want_links )
+    @{ $self->{_links} }
 	or do {
 	$self->_pass();
 	return wantarray ? ( 0, 1, 0 ) : 0;
     };
-    $self->{_links} = \@links;
 
     my $errors = 0;
 
-    foreach my $link ( @links ) {
+    foreach my $link ( @{ $self->{_links} } ) {
 	my $code = $self->can( "_handle_$link->[1]{type}" )
-	    or Carp::confess( "TODO - link type $link->[1]{type} not supported" );
+	    or Carp::confess(
+	    "TODO - link type $link->[1]{type} not supported" );
 	$errors += $code->( $self, $link );
     }
 
@@ -329,6 +353,11 @@ sub pod_file_ok {
 	$self->{_test}{fail};
 }
 
+sub require_installed {
+    my ( $self ) = @_;
+    return $self->{require_installed};
+}
+
 sub _user_agent {
     my ( $self ) = @_;
     return( $self->{_user_agent} ||= HTTP::Tiny->new(
@@ -336,11 +365,6 @@ sub _user_agent {
 	) );
 }
 
-sub agent {
-    my ( $self ) = @_;
-    return $self->{agent};
-}
-
 sub _pass {
     my ( $self, @msg ) = @_;
     @msg
@@ -368,6 +392,13 @@ sub _skip {
     return 0;
 }
 
+sub _any_errata_seen {
+    my ( $self, $file ) = @_;
+    $file = defined $file ? "File $file" : $self->{_file_name};
+    $TEST->diag( "$file contains POD errors" );
+    return;
+}
+
 # This method formats test messages. It is PRIVATE to this package, and
 # can be changed or revoked without notice.
 sub __build_test_msg {
@@ -376,58 +407,13 @@ sub __build_test_msg {
     if ( ARRAY_REF eq ref $msg[0] ) {
 	my $link = shift @msg;
 	my $text = "link L<$link->[1]{raw}>";
-	if ( defined $link->[1]{paragraph_start_line} ) {
-	    push @prefix, "line $link->[1]{paragraph_start_line}ff.";
-	    $text =~ s/ ( . ) /uc $1/smxe;
-	}
+	defined $link->[1]{line_number}
+	    and push @prefix, "line $link->[1]{line_number}";
 	push @prefix, $text;
     }
     return join ' ', @prefix, join '', @msg;
 }
 
-# Build the section hash. This has a key for each section found in the
-# parse tree, with a true value for that key. The return is a reference
-# to said hash.
-sub _build_section_hash {
-    my ( $self, $root ) = @_;
-    return {
-	map { $_->[2] => 1 }
-	$self->_extract_nodes( \&_want_sections, $root ) };
-}
-
-# my @nodes = $self->_extract_nodes( $want, $node )
-#
-# This subroutine extracts all subnodes of the given node that pass the
-# test specified by $want. The arguments are:
-#
-# $want is a code reference called with arguments $self and the subnode
-#   being considered. It should return the subnode itself if that is
-#   wanted, or a false value otherwise. The default always returns the
-#   subnode.
-# $node is the node from which subnodes are to be extracted. If
-#   unspecified the root of the latest parse will be used. Note that
-#   this argument will be returned if it passes the $want check.
-#
-sub _extract_nodes {
-    my ( $self, $want, $node, $para_line ) = @_;
-
-    $want ||= sub { return $_[1] };
-    defined $node
-	or $node = $self->{_root};
-
-    ref $node
-	or return;
-
-    $para_line = $node->[1]{start_line} || $para_line;
-
-    # The grep() below is paranoia based on the amount of pain incurred
-    # in finding that I needed to check for a defined value rather than
-    # a true value for the $node argument.
-    return (
-	$want->( $self, $node, $para_line ), map { $self->_extract_nodes(
-	    $want, $_, $para_line ) } grep { defined } @$node[ 2 .. $#$node ] );
-}
-
 # Get the information on installed documentation. If the doc is found
 # the return is a reference to a hash containing key {file}, value the
 # path name to the file containing the documentation. This works both
@@ -486,7 +472,6 @@ sub _handle_pod {
 
     } elsif ( my $section = $link->[1]{section} ) {
 	# Internal links (no {to})
-	$self->{_section} ||= $self->_build_section_hash();
 	$self->{_section}{$section}
 	    and return 0;
 	return $self->_fail( $link, 'links to unknown section' );
@@ -500,8 +485,10 @@ sub _handle_pod {
 
 sub _check_external_pod_info {
     my ( $self, $link ) = @_;
-    my $module = $link->[1]{to};
-    my $section = $link->[1]{section};
+
+    # Stringify overloaded objects
+    my $module = $link->[1]{to} ? "$link->[1]{to}" : undef;
+    my $section = $link->[1]{section} ?  "$link->[1]{section}" : undef;
 
     # If there is no section info it might be a Perl builtin. Return
     # success if it is.
@@ -510,13 +497,6 @@ sub _check_external_pod_info {
 	    and return 0;
     }
 
-    # NOTE: I have an example of this 'if' returning false when perhaps
-    # it should return true and then fail the link. The example is
-    # L<Pod::Perldoc|Pod::Perldoc>. Under Perl 5.8.9 ships with
-    # Pod::Perldoc 3.14, which is undocumented. But because the
-    # documentation is no found we assume the module is not installed,
-    # and fall through to consulting the module index, which succeeds.
-    #
     # If it is installed, handle it
     if ( my $data = $self->{_cache}{installed}{$module} ||=
 	$self->_get_installed_doc_info( $module ) ) {
@@ -532,22 +512,18 @@ sub _check_external_pod_info {
 	    and $self->_fail(
 	    "$module is installed but undocumented" );
 
-	# If it is in fact an installed module AND there is no section,
-	# we can return success.
-	$section
+	# If we get this far it is an installed module with
+	# documentation. We can return success at this point unless the
+	# link specifies a section AND we are checking them. We test the
+	# link rather than the section name because the latter could be
+	# '0'.
+	$link->[1]{section}
+	    and $self->check_external_sections()
 	    or return 0;
 
 	# Find and parse the section info if needed.
-	unless ( $data->{section} ) {
-
-	    my $parser = Pod::Simple::SimpleTree->new();
-	    $parser->parse_file( $data->{file} );
-	    $parser->any_errata_seen()
-		and $TEST->diag(
-		"File $data->{file} contains POD errors" );
-	    $data->{section} = $self->_build_section_hash(
-		$parser->root() );
-	}
+	$data->{section} ||= My_Parser->new()->run( $data->{file},
+	    \&_any_errata_seen, $self, "File $data->{file}" );
 
 	$data->{section}{$section}
 	    and return 0;
@@ -558,10 +534,15 @@ sub _check_external_pod_info {
     # If there is no section, it might be a man page, even though the
     # parser did not parse it as one
     unless ( $section ) {
-	$self->_is_man_page( $module )
+	$self->_is_man_page( $link )
 	    and return 0;
     }
 
+    # If we're requiring links to be to installed modules, flunk now.
+    $self->require_installed()
+	and return $self->_fail( $link,
+	'links to module that is not installed' );
+
     # It's not installed on this system, but it may be out there
     # somewhere
 
@@ -599,12 +580,12 @@ sub _get_module_index {
 
 # In all of the module index getters, the return is either nothing at
 # all (for inability to use this indexing mechanism) or a refererence to
-# to an array. Element [0] of the array is a reference a piece of code
-# that takes the module name as its only argument, and returns a true
-# value if that module exists and a false value otherwise. Element [1]
-# of the array is a Perl time that is characteristic of the information
-# in the index (typically the revision date of the underlying file if
-# that's the way the index works).
+# an array. Element [0] of the array is a reference a piece of code that
+# takes the module name as its only argument, and returns a true value
+# if that module exists and a false value otherwise. Element [1] of the
+# array is a Perl time that is characteristic of the information in the
+# index (typically the revision date of the underlying file if that's
+# the way the index works).
 
 # NOTE that Test::Pod::LinkCheck loads CPAN and then messes with it to
 # try to prevent it from initializing itself. After trying this and
@@ -786,30 +767,87 @@ sub _is_perl_file {
     }
 }
 
-sub _want_links {
-    my ( undef, $node, $para_line ) = @_;
-    'L' eq $node->[0]
-	or return;
-    if ( defined $para_line ) {
-	$node->[1]{paragraph_start_line} ||= $para_line;
-    }
-    return $node;
+package		## no critic (ProhibitMultiplePackages)
+My_Parser;	# Cargo cult to hide package from toolchain.
+
+use Pod::Simple::PullParser;	# Core since 5.9.3 (part of Pod::Simple)
+
+@My_Parser::ISA = qw{ Pod::Simple::PullParser };
+
+sub new {
+    my ( $class ) = @_;
+    my $self = $class->SUPER::new();
+    $self->preserve_whitespace( 1 );
+    return $self;
 }
 
-sub _want_sections {
-    my ( undef, $node ) = @_;
-    $node->[0] =~ m/ \A head \d+ \z /smx
-	or return;
+sub links {
+    my ( $self ) = @_;
+    return @{ $self->{My_Parser}{links} };
+}
 
-    # The following rigamarole is to flatten section names that contain
-    # formatting codes (e.g. 'foo(I<bar>)'). The formatting codes get
-    # lost in the process, but it's how the links themselves come out of
-    # the parse, so if we're wrong we are at least consistent.
-    my $ls = Pod::Simple::LinkSection->new(
-	@{ $node }[ 2 .. $#$node ] );
-    @{ $node }[ 2 .. $#$node ] = "$ls";
+sub run {
+    my ( $self, $source, $err, @err_arg ) = @_;
+	defined $source
+	    and $self->set_source( $source );
+    my $attr = $self->_attr();
+    @{ $attr }{ qw{ line links sections } } = ( 1, [], {} );
+    while ( my $token = $self->get_token() ) {
+	if ( my $code = $self->can( '__token_' . $token->type() ) ) {
+	    $code->( $self, $token );
+	}
+    }
+    $err
+	and $self->any_errata_seen()
+	and $err->( @err_arg );
+    return wantarray ?
+	( $attr->{sections}, $attr->{links} ) :
+	$attr->{sections};
+}
 
-    return $node;
+sub sections {
+    my ( $self ) = @_;
+    return @{ $self->_attr()->{sections} };
+}
+
+sub _attr {
+    my ( $self ) = @_;
+    return $self->{ ( __PACKAGE__ ) } ||= {};
+}
+
+sub __token_start {
+    my ( $self, $token ) = @_;
+    my $attr = $self->_attr();
+    if ( defined( my $line = $token->attr( 'start_line' ) ) ) {
+	$attr->{line} = $line;
+    }
+    my $tag = $token->tag();
+    if ( 'L' eq $tag ) {
+	$token->attr( line_number => $self->{My_Parser}{line} );
+	push @{ $attr->{links} }, [ @{ $token }[ 1 .. $#$token ] ];
+    } elsif ( $tag =~ m/ \A head [1-9] \z /smx ) {
+	$attr->{text} = '';
+    }
+    return;
+}
+
+sub __token_text {
+    my ( $self, $token ) = @_;
+    my $attr = $self->_attr();
+    my $text = $token->text();
+    $attr->{line} += $text =~ tr/\n//;
+    $attr->{text} .= $text;
+    return;
+}
+
+sub __token_end {
+    my ( $self, $token ) = @_;
+    my $attr = $self->_attr();
+    my $tag = $token->tag();
+    if ( $tag =~ m/ \A head [1-9] \z /smx ) {
+	$attr->{sections}{ delete $attr->{text} } = 1;
+    }
+    return;
 }
 
 1;
@@ -965,10 +1003,21 @@ This argument is the user agent string to use for web access.
 
 The default is that of L<HTTP::Tiny|HTTP::Tiny>.
 
+=item check_external_sections
+
+This Boolean argument is true if the sections of links outside the
+current Pod are to be checked. If it is false, such sections are not
+checked, and the link is considered valid if the external Pod exists at
+all.
+
+The default is true.
+
 =item check_url
 
 This Boolean argument is true if C<url> links are to be checked, and
-false if not. The default is true.
+false if not.
+
+The default is true.
 
 =item ignore_url
 
@@ -1047,6 +1096,14 @@ indices.
 
 By default all indices are considered.
 
+=item require_installed
+
+This Boolean argument is true to disable the uninstalled module checks.
+This means links to modules not installed on the system will fail, even
+if the module exists.
+
+By default this is false.
+
 =back
 
 =head2 agent
@@ -1073,6 +1130,13 @@ If called in scalar context, this method returns the number of test
 failures encountered. If called in list context it return the number of
 failures, passes, and skipped tests, in that order.
 
+=head2 check_external_sections
+
+ $t->check_external_sections()
+     and say 'Sections in external links are checked';
+
+This method returns the value of the C<'check_url'> attribute.
+
 =head2 check_url
 
  $t->check_url() and say 'URL links are checked';
@@ -1104,6 +1168,13 @@ If called in scalar context, this method returns the number of test
 failures encountered. If called in list context it return the number of
 failures, passes, and skipped tests, in that order.
 
+=head2 require_installed
+
+ $t->require_installed()
+    and say 'All POD links must be to installed modules';
+
+This method returns the value of the C<'require_installed'> attribute.
+
 =head1 SEE ALSO
 
 L<Test::Pod::LinkCheck|Test::Pod::LinkCheck> by Apocalypse checks all
